<?php

namespace App\Http\Controllers;


use App\Models\User;
use Carbon\Carbon;
use GuzzleHttp\Client;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Cache;
use Illuminate\Support\Facades\Config;
use EasyWeChat;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Log;
use Overtrue\EasySms\EasySms;
use Overtrue\EasySms\PhoneNumber;

class LoginController extends Controller
{

    /*
     * 获取openid
     */
    public function getUserOpenid($Code){
        $client=new Client();
        $AppId=config('wechat.official_account.default.app_id');
        $Secret=config('wechat.official_account.default.secret');
        $url='https://api.weixin.qq.com/sns/oauth2/access_token?appid='.$AppId.'&secret='.$Secret.'&code='.$Code.'&grant_type=authorization_code';
        $data=$client->request('get',$url);
        $da=$data->getBody()->getContents();
        $da=json_decode($da,true);
        $da=(array)$da;
        return $da;
    }

    /*
     * 获取微信用户信息
     */
    public function getWxUserInfo($openid,$access_token){
        $client=new Client();
        $url="https://api.weixin.qq.com/sns/userinfo";
        $array=[
            'query' => [
                'access_token' =>$access_token,
                'openid'=>$openid
            ]
        ];
        $data=$client->request('GET',$url,$array);
        $userInfo = json_decode($data->getBody()->getContents(),true);
        return $userInfo;
    }

    public function wechatLogin(Request $request){
        $Code=$request->code;
        $data=$this->getUserOpenid($Code);

        if(array_key_exists('errcode',$data) && $data['errcode'] == 40163){
            $da['openid'] = Cache::get('openid_'.$Code);
            $da['access_token'] = Cache::get('access_token_'.$Code);
            return $this->success($da);
        }
        if(isset($data['openid'])){
            $user=User::where('openid',$data['openid'])->first();
            if($user){//账号存在
                $token = Auth::guard('api')->fromUser($user);
                $da['token'] = $token;
                $da['openid'] = $data['openid'];
                //修改账号信息！！！！！！！！！！！！！！！！
                $userInfo=$this->getWxUserInfo($data['openid'],$data['access_token']);
                if(isset($userInfo['openid'])){
                    $user->avatar=$userInfo['headimgurl'];//$this->uploadAvatarImg($userInfo['headimgurl'])??
                    $user->nickname=$userInfo['nickname'];
                    $user->save();
                }
                return $this->success($da);
            }else{//账号不存在
                $da['openid'] = Cache::remember('openid_'.$Code,$data['expires_in'],function()use($data){
                    return $data['openid'];
                });
                $da['access_token'] = Cache::remember('access_token_'.$Code,$data['expires_in'],function()use($data){
                    return $data['access_token'];
                });
                return $this->success($da);
            }
        }else{
            return $this->error('450001','','您未关注微信公众号');
        }
    }

    public function login(Request $request, User $user)
    {
        Log::error('{-------登录--------}' . implode(',', $request->all()));
        $mobiles=mb_substr($request->mobile,0,3).'****'.mb_substr($request->mobile,'-4');
        $userss = User::where('mobile',$mobiles)->where('deleted_at','=',null)->get();
        $user=[];
        foreach ($userss as $k=>$v){
            $p=Crypt::decryptString($v->mobile_code);
            if ($p==$request->mobile){
                $user=User::find($v->id);
            }
        }
        if(empty($user)){
            return $this->error('450001','','账号不存在');
        }
        if ($user->status==1){
            return $this->error(700,'','账号已禁用');
        }
        if($request->has('verify_code')){
            $test_phone='15866666666';
            $test_verify='686313';
            if($request->input('mobile') != $test_phone || $request->input('verify_code') != $test_verify){
                //短信登录
                $code = $this->judge_code('verificationCode_' . MD5($request->input('code').'-'.$request->input('mobile')), $request->input('verify_code'));
                if ($code == 300) {
                    return $this->error(300,'','验证码失效');
                }
                if ($code == 400) {
                    return $this->error(300,'','验证码失效');
                }
            }
        }

        //获取用户信息
        if(empty($user->openid)){
            if ($request->has('openid') && $request->has('access_token')) {
                if(empty($request->openid)){
                    return $this->error('450001','','您未关注微信公众号,不能登陆');
                }
                $u_openid=User::where('openid',$request->openid)->first();
                if($u_openid){
                    return $this->error('450001','','您的账号已在其他微信登录！');
                }

                //获取用户微信信息
                $userInfo=$this->getWxUserInfo($request->openid,$request->access_token);
                if(isset($userInfo['openid'])){
                    $users['openid'] = $userInfo['openid'];
                    $users['unionid'] = $userInfo['unionid'];
                    $users['name'] = $userInfo['nickname'];
                    $users['avatar'] = $userInfo['headimgurl'];
                    User::where('id', $user->id)->update($users);
                }
            }else{
                return $this->error('450001','','获取微信系统信息失败,不能登录');
            }
        }else{
            if($user->openid != $request->openid){
                return $this->error('450001','','该账号已被微信昵称为'.$user->nickname.'的用户所绑定');
            }
        }
        $token = Auth::guard('api')->fromUser($user);
        $data = ['token' =>$token];
        Cache::forget('verificationCode_' . MD5($request->input('code').'-'.$request->input('mobile')));
        return $this->success($data);
    }
    /*判断验证码*/
    public function judge_code($verify_key, $verify_code)
    {
        $verifyData = Cache::get($verify_key);
        if (!$verifyData) {
            return 400;
        }
        if (!hash_equals($verifyData['code'], $verify_code)) {
            return 300;
        }
        return 200;
    }

    public function SendSms(Request $request){
//        if($request->mobile=='13949003369'){
//            return $this->success();
//        }
        $mobiles=mb_substr($request->mobile,0,3).'****'.mb_substr($request->mobile,'-4');
        $users = User::where('mobile',$mobiles)->get();
        $user=[];
        foreach ($users as $k=>$v){
            $p=Crypt::decryptString($v->mobile_code);
            if ($p==$request->mobile){
                $user=User::find($v->id);
            }
        }
        if (!$user){
            return $this->error(50214,$users,'不存在该账号！');
        }
        if ($user->status==1 || $user->deleted_at){
            return $this->error(50214,'','该账号已冻结！');
        }
        $sms = new EasySms(config('easysms'));
        $phone=$request->input('mobile');
        $time = 10;
        if(strlen($phone)==11){
            $code = mb_substr($phone, 3, 2) . mb_substr($phone, 8, 2) . mb_substr($phone, 6, 2);
        }else{
            $code = mb_substr($phone,3,2).mb_substr($phone, 1, 2) . mb_substr($phone, 6, 2);
        }
        $area_code='+86';
//        $number = new PhoneNumber($phone, $area_code);
//        try {
//            $result = $sms->send($number, [
//                'content' => "【大卫博士】您的验证码为{$code}，请在{$time}分钟内输入验证码！如非本人操作，请忽略此条短信。"
//            ]);
//        } catch (\Overtrue\EasySms\Exceptions\NoGatewayAvailableException $exception) {
//            $message = $exception->getException('qcloud')->getMessage();
//            Log::error('{短信发送错误------------}' . $message . '{---------------短信发送错误}');
//            return ['code'=>300,'message'=>$message];
//        }
        $key = 'verificationCode_' . MD5($area_code.'-'.$phone);
        $expiredAt = Carbon::now()->addMinutes(10);
        Cache::put($key, ['phone' => $phone, 'code' => $code], $expiredAt);   // 缓存验证码 10分钟过期。
        return $this->success(['key'=>$key,'code'=>$code]);
    }

    public function LoginSms(Request $request){
//        if ($request->input('mobile')==13949003369){
//            $user=User::find(1);
//            $token = Auth::guard('api')->fromUser($user);
//            $user->openid=$request->input('openid');
//            $user->save();
//            return $this->success(['token'=>$token]);
//        }
        $sms_code=$request->input('sms_code');
        $sms_key=$request->input('sms_key');
        if (!Cache::get($sms_key)){
            return $this->error(20014,'','验证码已过期！');
        }
        $sms=Cache::get($sms_key);
        if ($sms['code']!=$sms_code){
            return $this->error(20014,'','验证码错误 ！');
        }
        $mobiles=mb_substr($request->mobile,0,3).'****'.mb_substr($request->mobile,'-4');
        $users = User::where('mobile',$mobiles)->where('deleted_at','=',null)->get();
        $user=[];
        foreach ($users as $k=>$v){
            $p=Crypt::decryptString($v->mobile_code);
            if ($p==$request->mobile){
                $user=User::find($v->id);
            }
        }
        $user->openid=$request->input('openid');
        $user->save();
        $token = Auth::guard('api')->fromUser($user);
        return $this->success(['token'=>$token]);
    }
}
