<?php

namespace App\Http\Middleware;

use Tymon\JWTAuth\Http\Middleware\BaseMiddleware;
use Tymon\JWTAuth\Exceptions\TokenExpiredException;
use Tymon\JWTAuth\Exceptions\JWTException;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;
use Closure;
use Auth;
use Illuminate\Http\Request;
class CheckTokenMiddleware extends BaseMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  \Closure  $next
     * @return mixed
     */
    public function handle($request, Closure $next)
    {
        // 检查此次请求中是否带有 token，如果没有则抛出异常。
        try{
            $this->checkForToken($request);
        }catch (\Exception $exception){

            return response()->json([
                'code'=>50000,
                'status'  => 'fail',
                'message' => '请先进行登录',
                'data'    => $data ?? null,
                'error'  => null,
            ]);
        }

        // 使用 try 包裹，以捕捉 token 过期所抛出的 TokenExpiredException  异常
        try {
            // 检测用户的登录状态，如果正常则通过
            if ($this->auth->parseToken()->authenticate()) {
                return $next($request);
            }
            return response()->json([
                'code'=>50000,
                'msg'=>'请先登录',
            ]);
        } catch (TokenExpiredException $exception) {
            try {
                return response()->json([
                    'code'=>50001,
                    'msg'=>'登录失效',
                ]);
            } catch (JWTException $exception) {
                return response()->json([
                    'code'=>50001,
                    'msg'=>'登录失效，请重新登陆',
                ]);
            }
        }
    }
}
